@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2008-2010 Jens Vagelpohl and Contributors. All Rights Reserved.
+# Copyright (c) 2008-2012 Jens Vagelpohl and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
 # Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
@@ -14,8 +14,6 @@
 
 Instances of this class offer a simplified API to do searches, insertions, 
 deletions or modifications.
-
-$Id$
 """
 
 import codecs
@@ -60,9 +58,7 @@ class LDAPConnection(object):
         self.ldap_encoding = 'UTF-8'
         self.api_encoding = 'iso-8859-15'
 
-        if isinstance(bind_dn, unicode):
-            bind_dn = bind_dn.encode(self.api_encoding)
-        self.bind_dn = escape_dn(bind_dn).decode(self.api_encoding)
+        self.bind_dn = bind_dn
         self.bind_pwd = bind_pwd
         self.read_only = read_only
         self.c_factory = c_factory
@@ -115,10 +111,11 @@ class LDAPConnection(object):
             raise RuntimeError('No servers defined')
 
         if bind_dn is None:
-            bind_dn = self._encode_bind_dn()
-            bind_pwd = self.bind_pwd
+            bind_dn = escape_dn(self._encode_incoming(self.bind_dn))
+            bind_pwd = self._encode_incoming(self.bind_pwd)
         else:
-            bind_dn = escape_dn(self._encode_bind_dn(bind_dn))
+            bind_dn = escape_dn(self._encode_incoming(bind_dn))
+            bind_pwd = self._encode_incoming(bind_pwd)
 
         conn = self._getConnection()
         if conn is None:
@@ -148,8 +145,8 @@ class LDAPConnection(object):
 
         last_bind = getattr(conn, '_last_bind', None)
         if ( not last_bind or
-             ( last_bind[1][0] != bind_dn and 
-               last_bind[1][1] != bind_pwd ) ):
+             last_bind[1][0] != bind_dn or
+             last_bind[1][1] != bind_pwd ):
             conn.simple_bind_s(bind_dn, bind_pwd)
 
         return conn
@@ -186,6 +183,14 @@ class LDAPConnection(object):
 
         return connection
 
+    def disconnect(self):
+        """ Unbind the connection and invalidate the cache
+        """
+        conn = self._getConnection()
+        if conn is not None:
+            connection_cache.invalidate(self.hash)
+            conn.unbind_s()
+
     def search( self
               , base
               , scope=ldap.SCOPE_SUBTREE
@@ -194,6 +199,7 @@ class LDAPConnection(object):
               , convert_filter=True
               , bind_dn=None
               , bind_pwd=None
+              , raw=False
               ):
         """ Search for entries in the database
         """
@@ -229,15 +235,18 @@ class LDAPConnection(object):
                 # 'items' not found on rec_dict
                 continue
 
-            for key, value in items:
-                if key.lower() not in BINARY_ATTRIBUTES:
-                    if not isinstance(value, str):
-                        for i in range(len(value)):
-                            value[i] = self._encode_outgoing(value[i])
-                    else:
-                        rec_dict[key] = self._encode_outgoing(value)
+            if raw:
+                rec_dict['dn'] = rec_dn
+            else:
+                for key, value in items:
+                    if key.lower() not in BINARY_ATTRIBUTES:
+                        if not isinstance(value, basestring):
+                            for i in range(len(value)):
+                                value[i] = self._encode_outgoing(value[i])
+                        else:
+                            rec_dict[key] = self._encode_outgoing(value)
 
-            rec_dict['dn'] = self._encode_outgoing(rec_dn)
+                rec_dict['dn'] = self._encode_outgoing(rec_dn)
 
             result['results'].append(rec_dict)
             result['size'] += 1
@@ -309,6 +318,7 @@ class LDAPConnection(object):
                          , scope=ldap.SCOPE_BASE
                          , bind_dn=bind_dn
                          , bind_pwd=bind_pwd
+                         , raw=True
                          )
         attrs = attrs and attrs or {}
         cur_rec = res['results'][0]
@@ -330,10 +340,10 @@ class LDAPConnection(object):
                     mod_list.append((ldap.MOD_REPLACE, key, values))
                 elif cur_rec.has_key(key) and values in ([''], []):
                     mod_list.append((ldap.MOD_DELETE, key, None))
-            elif values == [''] and mod_type in (ldap.MOD_ADD, ldap.MOD_DELETE):
+            elif mod_type in (ldap.MOD_ADD, ldap.MOD_DELETE) and values == ['']:
                 continue
             elif ( mod_type == ldap.MOD_DELETE and
-                   set(values) != set(cur_rec.get(key, [])) ):
+                   set(values).difference(set(cur_rec.get(key, []))) ):
                 continue
             else:
                 mod_list.append((mod_type, key, values))
@@ -349,13 +359,14 @@ class LDAPConnection(object):
                 raw_rdn = [raw_rdn]
             new_rdn = raw_rdn[0]
 
-            if new_rdn and new_rdn != cur_rec.get(rdn_attr)[0]:
+            if new_rdn:
                 rdn_value = self._encode_incoming(new_rdn)
-                dn_parts[0] = [(rdn_attr, rdn_value, 1)]
-                raw_utf8_rdn = self._encode_incoming(rdn_attr + '=' + rdn_value)
-                new_rdn = escape_dn(raw_utf8_rdn)
-                connection.modrdn_s(dn, new_rdn)
-                dn = dn2str(dn_parts)
+                if rdn_value != cur_rec.get(rdn_attr)[0]:
+                    dn_parts[0] = [(rdn_attr, rdn_value, 1)]
+                    raw_utf8_rdn = rdn_attr + '=' + rdn_value
+                    new_rdn = escape_dn(raw_utf8_rdn)
+                    connection.modrdn_s(dn, new_rdn)
+                    dn = dn2str(dn_parts)
 
             if mod_list:
                 connection.modify_s(dn, mod_list)
@@ -377,7 +388,8 @@ class LDAPConnection(object):
         if ldapurl.isLDAPUrl(ldap_url):
             conn_str = ldapurl.LDAPUrl(ldap_url).initializeUrl()
             conn = self._connect(conn_str)
-            conn.simple_bind_s(self._encode_bind_dn(), self.bind_pwd)
+            conn.simple_bind_s(self._encode_incoming(self.bind_dn),
+                               self._encode_incoming(self.bind_pwd))
             return conn
         else:
             raise ldap.CONNECT_ERROR, 'Bad referral "%s"' % str(exception)
@@ -411,10 +423,11 @@ class LDAPConnection(object):
 
         else:
             if self.api_encoding != self.ldap_encoding:
-                value = value.decode(self.api_encoding)
+                if self.api_encoding:
+                    value = value.decode(self.api_encoding)
 
-                if self.ldap_encoding:
-                    value = value.encode(self.ldap_encoding)
+                    if self.ldap_encoding:
+                        value = value.encode(self.ldap_encoding)
         
         return value
 
@@ -422,7 +435,7 @@ class LDAPConnection(object):
         """ Encode a string value to the API encoding
 
         - if "value" is unicode, it will be encoded to self.api_encoding, but
-          only if self.ldap_encoding is set.
+          only if self.api_encoding is set.
         - if "value" is not unicode, it is assumed to be encoded as 
           self.ldap_encoding. It is decoded and encoded to self.api_encoding
           if self.api_encoding is set, unless self.ldap_encoding and 
@@ -438,21 +451,10 @@ class LDAPConnection(object):
 
         else:
             if self.api_encoding != self.ldap_encoding:
-                value = value.decode(self.ldap_encoding)
+                if self.ldap_encoding:
+                    value = value.decode(self.ldap_encoding)
 
-                if self.api_encoding:
-                    value = value.encode(self.api_encoding)
+                    if self.api_encoding:
+                        value = value.encode(self.api_encoding)
         
         return value
-
-    def _encode_bind_dn(self, dn=None):
-        """ Make sure the stored bind DN is encoded correctly upon access
-        """
-        bind_dn = dn or getattr(self, 'bind_dn', '')
-
-        if not isinstance(bind_dn, unicode):
-            bind_dn = bind_dn.decode(self.api_encoding)
-
-        return bind_dn.encode(self.ldap_encoding)
-
-
