@@ -156,8 +156,3 @@ class ConnectionSearchTests(LDAPConnectionTests):
                           }
                         )
 
-
-def test_suite():
-    import sys
-    return unittest.findTestCases(sys.modules[__name__])
-
